@@ -9,7 +9,7 @@ from typing import Any, Dict, List, Optional
 from fastmcp import FastMCP
 import os
 
-from mcp_airflow_api.functions import airflow_request, read_prompt_template, parse_prompt_sections, get_current_time_context
+from mcp_airflow_api.functions import airflow_request, read_prompt_template, parse_prompt_sections, get_current_time_context, list_dags_internal, get_dag_detailed_info
 # Handle both direct execution and module execution
 # try:
 #     from .functions import airflow_request, read_prompt_template, parse_prompt_sections, get_current_time_context
@@ -110,110 +110,7 @@ def list_dags(limit: int = 20,
         - has_more_pages: Boolean indicating if more pages are available
         - next_offset: Suggested offset for next page (if has_more_pages is True)
     """
-    # Helper: server-side filtering by ID and display name
-    def _filter_dags(dag_list):
-        results = dag_list
-        if id_contains:
-            key = id_contains.lower()
-            results = [d for d in results if key in d.get("dag_id", "").lower()]
-        if name_contains:
-            key = name_contains.lower()
-            results = [d for d in results if key in (d.get("dag_display_name") or "").lower()]
-        return results
-
-    # If fetch_all=True, loop through pages to collect all DAGs
-    if fetch_all:
-        all_dags = []
-        current_offset = offset
-        total_entries = None
-        pages_fetched = 0
-        while True:
-            # recursive call without fetch_all to fetch one page
-            result = list_dags(limit=limit, offset=current_offset)
-            page_dags = result.get("dags", [])
-            all_dags.extend(page_dags)
-            pages_fetched += 1
-            total_entries = result.get("total_entries", 0)
-            if not result.get("has_more_pages", False) or not page_dags:
-                break
-            current_offset = result.get("next_offset", current_offset + limit)
-        # apply filters
-        filtered = _filter_dags(all_dags)
-        return {
-            "dags": filtered,
-            "total_entries": len(filtered),
-            "pages_fetched": pages_fetched,
-            "limit": limit,
-            "offset": offset
-        }
-    # Default: paginated fetch
-    params = []
-    params.append(f"limit={limit}")
-    if offset > 0:
-        params.append(f"offset={offset}")
-    
-    query_string = "&".join(params) if params else ""
-    endpoint = f"/dags?{query_string}" if query_string else "/dags"
-    
-    resp = airflow_request("GET", endpoint)
-    resp.raise_for_status()
-    response_data = resp.json()
-    dags = response_data.get("dags", [])
-    dag_list = []
-    for dag in dags:
-        # Extract schedule interval info
-        schedule_info = dag.get("schedule_interval")
-        if isinstance(schedule_info, dict) and schedule_info.get("__type") == "CronExpression":
-            schedule_display = schedule_info.get("value", "Unknown")
-        elif schedule_info:
-            schedule_display = str(schedule_info)
-        else:
-            schedule_display = None
-        
-        dag_info = {
-            "dag_id": dag.get("dag_id"),
-            "dag_display_name": dag.get("dag_display_name"),
-            "description": dag.get("description"),
-            "is_active": dag.get("is_active"),
-            "is_paused": dag.get("is_paused"),
-            "schedule_interval": schedule_display,
-            "max_active_runs": dag.get("max_active_runs"),
-            "max_active_tasks": dag.get("max_active_tasks"),
-            "owners": dag.get("owners"),
-            "tags": [t.get("name") for t in dag.get("tags", [])],
-            "next_dagrun": dag.get("next_dagrun"),
-            "next_dagrun_data_interval_start": dag.get("next_dagrun_data_interval_start"),
-            "next_dagrun_data_interval_end": dag.get("next_dagrun_data_interval_end"),
-            "last_parsed_time": dag.get("last_parsed_time"),
-            "has_import_errors": dag.get("has_import_errors"),
-            "has_task_concurrency_limits": dag.get("has_task_concurrency_limits"),
-            "timetable_description": dag.get("timetable_description"),
-            "fileloc": dag.get("fileloc"),
-            "file_token": dag.get("file_token")
-        }
-        dag_list.append(dag_info)
-    
-    # Calculate pagination info and apply filters
-    total_entries = response_data.get("total_entries", len(dag_list))
-    has_more_pages = (offset + limit) < total_entries
-    next_offset = offset + limit if has_more_pages else None
-    filtered = _filter_dags(dag_list)
-    returned_count = len(filtered)
-    
-    return {
-        "dags": filtered,
-        "total_entries": total_entries,
-        "limit": limit,
-        "offset": offset,
-        "returned_count": returned_count,
-        "has_more_pages": has_more_pages,
-        "next_offset": next_offset,
-        "pagination_info": {
-            "current_page": (offset // limit) + 1 if limit > 0 else 1,
-            "total_pages": ((total_entries - 1) // limit) + 1 if limit > 0 and total_entries > 0 else 1,
-            "remaining_count": max(0, total_entries - (offset + returned_count))
-        }
-    }
+    return list_dags_internal(limit, offset, fetch_all, id_contains, name_contains)
 
 @mcp.tool()
 def get_dag(dag_id: str) -> Dict[str, Any]:
@@ -226,30 +123,146 @@ def get_dag(dag_id: str) -> Dict[str, Any]:
     Returns:
         Comprehensive DAG details: dag_id, schedule_interval, start_date, owners, tags, description, etc.
     """
-    if not dag_id:
-        raise ValueError("dag_id must not be empty")
-    resp = airflow_request("GET", f"/dags/{dag_id}")
-    resp.raise_for_status()
-    dag = resp.json()
+    return get_dag_detailed_info(dag_id)
+
+@mcp.tool()
+def get_dags_detailed_batch(
+    limit: int = 100,
+    offset: int = 0,
+    fetch_all: bool = False,
+    id_contains: Optional[str] = None,
+    name_contains: Optional[str] = None,
+    is_active: Optional[bool] = None,
+    is_paused: Optional[bool] = None
+) -> Dict[str, Any]:
+    """
+    [Tool Role]: Retrieves detailed information for multiple DAGs in batch with get_dag() level detail plus latest run information.
+    
+    This tool combines list_dags() filtering with get_dag() detailed information retrieval,
+    providing comprehensive DAG details AND latest execution information for multiple DAGs in a single response.
+    Each DAG entry includes both static configuration details and dynamic runtime information.
+    
+    Args:
+        limit: Maximum number of DAGs to process (default: 100)
+               - Use higher values (500-1000) for large environments
+               - Ignored when fetch_all=True
+        offset: Number of DAGs to skip for pagination (default: 0)
+        fetch_all: If True, fetches all DAGs regardless of limit/offset (default: False)
+        id_contains: Filter DAGs by ID containing this string (optional)
+        name_contains: Filter DAGs by display name containing this string (optional)
+        is_active: Filter by active status - True/False (optional)
+        is_paused: Filter by paused status - True/False (optional)
+    
+    Usage Examples:
+        - All unpaused DAGs with full details and latest runs: get_dags_detailed_batch(fetch_all=True, is_paused=False)
+        - Active, unpaused DAGs only: get_dags_detailed_batch(is_active=True, is_paused=False)
+        - DAGs containing "example": get_dags_detailed_batch(id_contains="example", limit=50)
+        - Paginated batch: get_dags_detailed_batch(limit=100, offset=200)
+    
+    Returns:
+        Dictionary containing:
+        - dags_detailed: List of detailed DAG objects with:
+          * All get_dag() fields (dag_id, schedule_interval, start_date, owners, tags, etc.)
+          * latest_dag_run: Most recent execution information (run_id, state, start_date, end_date, etc.)
+        - total_processed: Number of DAGs successfully processed
+        - total_available: Total number of DAGs matching initial filters
+        - processing_stats: Success/failure counts and error details
+        - applied_filters: Summary of filters applied
+        - pagination_info: Current page info and remaining counts
+    """
+    # First get the list of DAGs with basic filtering using helper function
+    dag_list_result = list_dags_internal(
+        limit=limit, 
+        offset=offset, 
+        fetch_all=fetch_all,
+        id_contains=id_contains,
+        name_contains=name_contains
+    )
+    
+    dags_basic = dag_list_result.get("dags", [])
+    detailed_dags = []
+    success_count = 0
+    error_count = 0
+    errors = []
+    skipped_count = 0
+    
+    for dag_basic in dags_basic:
+        dag_id = dag_basic.get("dag_id")
+        if not dag_id:
+            skipped_count += 1
+            continue
+            
+        # Apply additional filters that require detailed DAG info
+        if is_active is not None and dag_basic.get("is_active") != is_active:
+            skipped_count += 1
+            continue
+        if is_paused is not None and dag_basic.get("is_paused") != is_paused:
+            skipped_count += 1
+            continue
+            
+        try:
+            # Get detailed information for this DAG using helper function
+            detailed_dag = get_dag_detailed_info(dag_id)
+            
+            # Also get the latest DAG run information for comprehensive details
+            try:
+                latest_run_resp = airflow_request("GET", f"/dags/{dag_id}/dagRuns?limit=1&order_by=-execution_date")
+                latest_run_resp.raise_for_status()
+                latest_runs = latest_run_resp.json().get("dag_runs", [])
+                
+                if latest_runs:
+                    latest_run = latest_runs[0]
+                    detailed_dag["latest_dag_run"] = {
+                        "run_id": latest_run.get("run_id"),
+                        "run_type": latest_run.get("run_type"),
+                        "state": latest_run.get("state"),
+                        "execution_date": latest_run.get("execution_date"),
+                        "start_date": latest_run.get("start_date"),
+                        "end_date": latest_run.get("end_date"),
+                        "data_interval_start": latest_run.get("data_interval_start"),
+                        "data_interval_end": latest_run.get("data_interval_end"),
+                        "external_trigger": latest_run.get("external_trigger"),
+                        "conf": latest_run.get("conf"),
+                        "note": latest_run.get("note")
+                    }
+                else:
+                    detailed_dag["latest_dag_run"] = None
+            except Exception:
+                # If we can't get latest run info, don't fail the whole operation
+                detailed_dag["latest_dag_run"] = None
+                
+            detailed_dags.append(detailed_dag)
+            success_count += 1
+        except Exception as e:
+            error_count += 1
+            errors.append({
+                "dag_id": dag_id,
+                "error": str(e)
+            })
+    
     return {
-        "dag_id": dag.get("dag_id"),
-        "dag_display_name": dag.get("dag_display_name"),
-        "description": dag.get("description"),
-        "schedule_interval": dag.get("schedule_interval"),
-        "start_date": dag.get("start_date"),
-        "end_date": dag.get("end_date"),
-        "is_active": dag.get("is_active"),
-        "is_paused": dag.get("is_paused"),
-        "owners": dag.get("owners"),
-        "tags": [t.get("name") for t in dag.get("tags", [])],
-        "catchup": dag.get("catchup"),
-        "max_active_runs": dag.get("max_active_runs"),
-        "max_active_tasks": dag.get("max_active_tasks"),
-        "has_task_concurrency_limits": dag.get("has_task_concurrency_limits"),
-        "has_import_errors": dag.get("has_import_errors"),
-        "next_dagrun": dag.get("next_dagrun"),
-        "next_dagrun_data_interval_start": dag.get("next_dagrun_data_interval_start"),
-        "next_dagrun_data_interval_end": dag.get("next_dagrun_data_interval_end")
+        "dags_detailed": detailed_dags,
+        "total_processed": success_count,
+        "total_available": dag_list_result.get("total_entries", 0),
+        "returned_count": len(detailed_dags),
+        "processing_stats": {
+            "success_count": success_count,
+            "error_count": error_count,
+            "skipped_count": skipped_count,
+            "errors": errors
+        },
+        "applied_filters": {
+            "id_contains": id_contains,
+            "name_contains": name_contains,
+            "is_active": is_active,
+            "is_paused": is_paused,
+            "limit": limit,
+            "offset": offset,
+            "fetch_all": fetch_all
+        },
+        "pagination_info": dag_list_result.get("pagination_info", {}),
+        "has_more_pages": dag_list_result.get("has_more_pages", False),
+        "next_offset": dag_list_result.get("next_offset")
     }
 
 @mcp.tool()
@@ -1671,6 +1684,272 @@ def prompt_template_section_prompt(section: Optional[str] = None) -> str:
         ])
     return get_prompt_template(section=section)
 
+#========================================================================================
+# Connection Management Functions
+#========================================================================================
+
+@mcp.tool()
+def list_connections(limit: int = 20, offset: int = 0, order_by: str = "connection_id") -> Dict[str, Any]:
+    """
+    [Tool Role]: List all connections in the Airflow instance with pagination support.
+    
+    Args:
+        limit: Maximum number of connections to return (default: 20)
+        offset: Number of connections to skip for pagination (default: 0)
+        order_by: Field to order by (default: "connection_id")
+                 Valid values: connection_id, conn_type, host, schema, login
+    
+    Returns:
+        List of connections with their configuration: connections, total_entries, limit, offset
+    """
+    # Build query parameters
+    params = [f"limit={limit}", f"offset={offset}"]
+    if order_by:
+        params.append(f"order_by={order_by}")
+    
+    query_string = "&".join(params)
+    resp = airflow_request("GET", f"/connections?{query_string}")
+    resp.raise_for_status()
+    data = resp.json()
+    
+    connections = []
+    for conn in data.get("connections", []):
+        # Only return safe connection info (exclude sensitive data like passwords)
+        conn_info = {
+            "connection_id": conn.get("connection_id"),
+            "conn_type": conn.get("conn_type"),
+            "description": conn.get("description"),
+            "host": conn.get("host"),
+            "schema": conn.get("schema"),
+            "login": conn.get("login"),
+            "port": conn.get("port"),
+            "is_encrypted": conn.get("is_encrypted"),
+            "is_extra_encrypted": conn.get("is_extra_encrypted"),
+            "extra": conn.get("extra")  # Note: may contain sensitive data
+        }
+        connections.append(conn_info)
+    
+    # Calculate pagination info
+    total_entries = data.get("total_entries", len(connections))
+    returned_count = len(connections)
+    has_more_pages = (offset + limit) < total_entries
+    next_offset = offset + limit if has_more_pages else None
+    
+    return {
+        "connections": connections,
+        "total_entries": total_entries,
+        "limit": limit,
+        "offset": offset,
+        "returned_count": returned_count,
+        "has_more_pages": has_more_pages,
+        "next_offset": next_offset,
+        "pagination_info": {
+            "current_page": (offset // limit) + 1 if limit > 0 else 1,
+            "total_pages": ((total_entries - 1) // limit) + 1 if limit > 0 and total_entries > 0 else 1,
+            "remaining_count": max(0, total_entries - (offset + returned_count))
+        }
+    }
+
+@mcp.tool()
+def get_connection(connection_id: str) -> Dict[str, Any]:
+    """
+    [Tool Role]: Get detailed information about a specific connection.
+    
+    Args:
+        connection_id: The connection ID to retrieve
+    
+    Returns:
+        Detailed connection information (excluding sensitive password data)
+    """
+    if not connection_id:
+        raise ValueError("connection_id must not be empty")
+    
+    resp = airflow_request("GET", f"/connections/{connection_id}")
+    resp.raise_for_status()
+    conn = resp.json()
+    
+    # Return connection info but mask sensitive data
+    return {
+        "connection_id": conn.get("connection_id"),
+        "conn_type": conn.get("conn_type"),
+        "description": conn.get("description"),
+        "host": conn.get("host"),
+        "schema": conn.get("schema"),
+        "login": conn.get("login"),
+        "port": conn.get("port"),
+        "is_encrypted": conn.get("is_encrypted"),
+        "is_extra_encrypted": conn.get("is_extra_encrypted"),
+        "extra": conn.get("extra"),  # Note: may contain sensitive data
+        "password": "[MASKED]" if conn.get("password") else None
+    }
+
+@mcp.tool()
+def create_connection(
+    connection_id: str,
+    conn_type: str,
+    description: Optional[str] = None,
+    host: Optional[str] = None,
+    login: Optional[str] = None,
+    password: Optional[str] = None,
+    schema: Optional[str] = None,
+    port: Optional[int] = None,
+    extra: Optional[str] = None
+) -> Dict[str, Any]:
+    """
+    [Tool Role]: Create a new connection in Airflow.
+    
+    Args:
+        connection_id: Unique identifier for the connection
+        conn_type: Type of connection (e.g., 'postgres', 'mysql', 'http', 's3', etc.)
+        description: Optional description of the connection
+        host: Host address for the connection
+        login: Login/username for authentication
+        password: Password for authentication (will be encrypted)
+        schema: Database schema or namespace
+        port: Port number for the connection
+        extra: Additional connection parameters as JSON string
+    
+    Returns:
+        Created connection information (excluding sensitive data)
+    """
+    if not connection_id or not conn_type:
+        raise ValueError("connection_id and conn_type are required")
+    
+    # Build request body
+    conn_data = {
+        "connection_id": connection_id,
+        "conn_type": conn_type
+    }
+    
+    # Add optional fields
+    optional_fields = {
+        "description": description,
+        "host": host,
+        "login": login,
+        "password": password,
+        "schema": schema,
+        "port": port,
+        "extra": extra
+    }
+    
+    for key, value in optional_fields.items():
+        if value is not None:
+            conn_data[key] = value
+    
+    resp = airflow_request("POST", "/connections", json=conn_data)
+    resp.raise_for_status()
+    conn = resp.json()
+    
+    return {
+        "connection_id": conn.get("connection_id"),
+        "conn_type": conn.get("conn_type"),
+        "description": conn.get("description"),
+        "host": conn.get("host"),
+        "login": conn.get("login"),
+        "schema": conn.get("schema"),
+        "port": conn.get("port"),
+        "is_encrypted": conn.get("is_encrypted"),
+        "is_extra_encrypted": conn.get("is_extra_encrypted"),
+        "extra": conn.get("extra"),
+        "password": "[MASKED]" if conn.get("password") else None,
+        "status": "created"
+    }
+
+@mcp.tool()
+def update_connection(
+    connection_id: str,
+    conn_type: Optional[str] = None,
+    description: Optional[str] = None,
+    host: Optional[str] = None,
+    login: Optional[str] = None,
+    password: Optional[str] = None,
+    schema: Optional[str] = None,
+    port: Optional[int] = None,
+    extra: Optional[str] = None
+) -> Dict[str, Any]:
+    """
+    [Tool Role]: Update an existing connection in Airflow.
+    
+    Args:
+        connection_id: The connection ID to update
+        conn_type: Type of connection (optional for updates)
+        description: Optional description of the connection
+        host: Host address for the connection
+        login: Login/username for authentication
+        password: Password for authentication (will be encrypted)
+        schema: Database schema or namespace
+        port: Port number for the connection
+        extra: Additional connection parameters as JSON string
+    
+    Returns:
+        Updated connection information (excluding sensitive data)
+    """
+    if not connection_id:
+        raise ValueError("connection_id must not be empty")
+    
+    # Build request body with only provided fields
+    conn_data = {}
+    
+    update_fields = {
+        "conn_type": conn_type,
+        "description": description,
+        "host": host,
+        "login": login,
+        "password": password,
+        "schema": schema,
+        "port": port,
+        "extra": extra
+    }
+    
+    for key, value in update_fields.items():
+        if value is not None:
+            conn_data[key] = value
+    
+    if not conn_data:
+        raise ValueError("At least one field must be provided for update")
+    
+    resp = airflow_request("PATCH", f"/connections/{connection_id}", json=conn_data)
+    resp.raise_for_status()
+    conn = resp.json()
+    
+    return {
+        "connection_id": conn.get("connection_id"),
+        "conn_type": conn.get("conn_type"),
+        "description": conn.get("description"),
+        "host": conn.get("host"),
+        "login": conn.get("login"),
+        "schema": conn.get("schema"),
+        "port": conn.get("port"),
+        "is_encrypted": conn.get("is_encrypted"),
+        "is_extra_encrypted": conn.get("is_extra_encrypted"),
+        "extra": conn.get("extra"),
+        "password": "[MASKED]" if conn.get("password") else None,
+        "status": "updated"
+    }
+
+@mcp.tool()
+def delete_connection(connection_id: str) -> Dict[str, Any]:
+    """
+    [Tool Role]: Delete a connection from Airflow.
+    
+    Args:
+        connection_id: The connection ID to delete
+    
+    Returns:
+        Confirmation of deletion: connection_id, status
+    """
+    if not connection_id:
+        raise ValueError("connection_id must not be empty")
+    
+    resp = airflow_request("DELETE", f"/connections/{connection_id}")
+    resp.raise_for_status()
+    
+    return {
+        "connection_id": connection_id,
+        "status": "deleted",
+        "message": f"Connection '{connection_id}' has been successfully deleted"
+    }
+
 #========================================================================================
 
 def main(argv: Optional[List[str]] = None):
@@ -1699,8 +1978,19 @@ def main(argv: Optional[List[str]] = None):
         logger.debug("Log level from environment: %s", logging.getLogger().level)
 
     # mcp.run(transport='stdio')
-    # mcp.run(transport='http', host="127.0.0.1", port=18000)
-    # mcp.run(transport="streamable-http", host="127.0.0.1", port=18000)
+    # mcp.run(transport='http', host="0.0.0.0", port=18000)
+    # mcp.run(transport="streamable-http", host="0.0.0.0", port=18000)
+    
+    # MCP_SERVER_PORT 환경변수가 있으면 streamable-http, 없으면 stdio
+    if os.getenv("MCP_SERVER_PORT"):
+        # MCP_SERVER_PORT 있음 → http transport
+        port = int(os.getenv("PORT", "18000"))
+        logger.info(f"Starting HTTP server on port {port} for smithery.ai")
+        mcp.run(transport="streamable-http", host="0.0.0.0", port=port)
+    else:
+        # MCP_SERVER_PORT 없음 → stdio transport
+        logger.info("Starting stdio transport for local usage")
+        mcp.run(transport='stdio')
 
 if __name__ == "__main__":
     main()
